@@ -225,22 +225,31 @@ struct ModuleSearchPathInner {
     kind: ModuleSearchPathKind,
 }
 
-#[derive(Copy, Clone, Debug, Eq, PartialEq, Hash)]
+/// Enumeration of the different kinds of search paths type checkers are expected to support.
+///
+/// N.B. Although we don't implement `Ord` for this enum, they are ordered in terms of the
+/// priority that we want to give these modules when resolving them.
+/// This is roughly [the order given in the typing spec], but typeshed's stubs
+/// for the standard library are moved higher up to match Python's semantics at runtime.
+///
+/// [the order given in the typing spec]: https://typing.readthedocs.io/en/latest/spec/distributing.html#import-resolution-ordering
+#[derive(Copy, Clone, Debug, Eq, PartialEq, Hash, is_macro::Is)]
 pub enum ModuleSearchPathKind {
-    // Project dependency
+    /// "Extra" paths provided by the user in a config file, env var or CLI flag.
+    /// E.g. mypy's `MYPYPATH` env var, or pyright's `stubPath` configuration setting
+    Extra,
+
+    /// Files in the project we're directly being invoked on
     FirstParty,
 
-    // e.g. site packages
-    ThirdParty,
-
-    // e.g. built-in modules, typeshed
+    /// The `stdlib` directory of typeshed (either vendored or custom)
     StandardLibrary,
-}
 
-impl ModuleSearchPathKind {
-    pub const fn is_first_party(self) -> bool {
-        matches!(self, Self::FirstParty)
-    }
+    /// Stubs or runtime modules installed in site-packages
+    SitePackagesThirdParty,
+
+    /// Vendored third-party stubs from typeshed
+    VendoredThirdParty,
 }
 
 #[derive(Debug, Eq, PartialEq)]
@@ -388,10 +397,82 @@ pub fn file_to_module(db: &dyn SemanticDb, file: FileId) -> QueryResult<Option<M
 //////////////////////////////////////////////////////
 
 /// Changes the module search paths to `search_paths`.
-pub fn set_module_search_paths(db: &mut dyn SemanticDb, search_paths: Vec<ModuleSearchPath>) {
+pub fn set_module_search_paths(db: &mut dyn SemanticDb, search_paths: ModuleResolutionInputs) {
     let jar: &mut SemanticJar = db.jar_mut();
 
-    jar.module_resolver = ModuleResolver::new(search_paths);
+    jar.module_resolver = ModuleResolver::new(search_paths.into_ordered_search_paths());
+}
+
+/// Struct for holding the various paths that are put together
+/// to create an `OrderedSearchPatsh` instance
+///
+/// - `extra_paths` is a list of user-provided paths
+///   that should take first priority in the module resolution.
+///   Examples in other type checkers are mypy's MYPYPATH environment variable,
+///   or pyright's stubPath configuration setting.
+/// - `workspace_root` is the root of the workspace,
+///   used for finding first-party modules
+/// - `site-packages` is the path to the user's `site-packages` directory,
+///   where third-party packages from ``PyPI`` are installed
+/// - `custom_typeshed` is a path to standard-library typeshed stubs.
+///   Currently this has to be a directory that exists on disk.
+///   (TODO: fall back to vendored stubs if no custom directory is provided.)
+#[derive(Debug)]
+pub struct ModuleResolutionInputs {
+    pub extra_paths: Vec<PathBuf>,
+    pub workspace_root: PathBuf,
+    pub site_packages: Option<PathBuf>,
+    pub custom_typeshed: Option<PathBuf>,
+}
+
+impl ModuleResolutionInputs {
+    /// Implementation of PEP 561's module resolution order
+    /// (with some small, deliberate, differences)
+    fn into_ordered_search_paths(self) -> OrderedSearchPaths {
+        let ModuleResolutionInputs {
+            extra_paths,
+            workspace_root,
+            site_packages,
+            custom_typeshed,
+        } = self;
+
+        OrderedSearchPaths(
+            extra_paths
+                .into_iter()
+                .map(|path| ModuleSearchPath::new(path, ModuleSearchPathKind::Extra))
+                .chain(std::iter::once(ModuleSearchPath::new(
+                    workspace_root,
+                    ModuleSearchPathKind::FirstParty,
+                )))
+                // TODO fallback to vendored typeshed stubs if no custom typeshed directory is provided by the user
+                .chain(custom_typeshed.into_iter().map(|path| {
+                    ModuleSearchPath::new(
+                        path.join(TYPESHED_STDLIB_DIRECTORY),
+                        ModuleSearchPathKind::StandardLibrary,
+                    )
+                }))
+                .chain(site_packages.into_iter().map(|path| {
+                    ModuleSearchPath::new(path, ModuleSearchPathKind::SitePackagesThirdParty)
+                }))
+                // TODO vendor typeshed's third-party stubs as well as the stdlib and fallback to them as a final step
+                .collect(),
+        )
+    }
+}
+
+const TYPESHED_STDLIB_DIRECTORY: &str = "stdlib";
+
+/// A resolved module resolution order, implementing PEP 561
+/// (with some small, deliberate differences)
+#[derive(Clone, Debug, Default, Eq, PartialEq)]
+struct OrderedSearchPaths(Vec<ModuleSearchPath>);
+
+impl Deref for OrderedSearchPaths {
+    type Target = [ModuleSearchPath];
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
 }
 
 /// Adds a module located at `path` to the resolver.
@@ -460,7 +541,7 @@ pub fn add_module(db: &mut dyn SemanticDb, path: &Path) -> Option<(Module, Vec<A
 #[derive(Default)]
 pub struct ModuleResolver {
     /// The search paths where modules are located (and searched). Corresponds to `sys.path` at runtime.
-    search_paths: Vec<ModuleSearchPath>,
+    search_paths: OrderedSearchPaths,
 
     // Locking: Locking is done by acquiring a (write) lock on `by_name`. This is because `by_name` is the primary
     // lookup method. Acquiring locks in any other ordering can result in deadlocks.
@@ -477,7 +558,7 @@ pub struct ModuleResolver {
 }
 
 impl ModuleResolver {
-    pub fn new(search_paths: Vec<ModuleSearchPath>) -> Self {
+    fn new(search_paths: OrderedSearchPaths) -> Self {
         Self {
             search_paths,
             modules: FxDashMap::default(),
@@ -689,13 +770,17 @@ impl PackageKind {
 
 #[cfg(test)]
 mod tests {
+    use std::io::{Cursor, Read};
     use std::num::NonZeroU32;
+    use std::path::{Path, PathBuf};
+
+    use zip::ZipArchive;
 
     use crate::db::tests::TestDb;
     use crate::db::SourceDb;
     use crate::module::{
         path_to_module, resolve_module, set_module_search_paths, ModuleKind, ModuleName,
-        ModuleSearchPath, ModuleSearchPathKind,
+        ModuleResolutionInputs, TYPESHED_STDLIB_DIRECTORY,
     };
     use crate::semantic::Dependency;
 
@@ -703,8 +788,9 @@ mod tests {
         temp_dir: tempfile::TempDir,
         db: TestDb,
 
-        src: ModuleSearchPath,
-        site_packages: ModuleSearchPath,
+        src: PathBuf,
+        custom_typeshed: PathBuf,
+        site_packages: PathBuf,
     }
 
     fn create_resolver() -> std::io::Result<TestCase> {
@@ -712,25 +798,31 @@ mod tests {
 
         let src = temp_dir.path().join("src");
         let site_packages = temp_dir.path().join("site_packages");
+        let custom_typeshed = temp_dir.path().join("typeshed");
 
         std::fs::create_dir(&src)?;
         std::fs::create_dir(&site_packages)?;
+        std::fs::create_dir(&custom_typeshed)?;
 
-        let src = ModuleSearchPath::new(src.canonicalize()?, ModuleSearchPathKind::FirstParty);
-        let site_packages = ModuleSearchPath::new(
-            site_packages.canonicalize()?,
-            ModuleSearchPathKind::ThirdParty,
-        );
+        let src = src.canonicalize()?;
+        let site_packages = site_packages.canonicalize()?;
+        let custom_typeshed = custom_typeshed.canonicalize()?;
 
-        let roots = vec![src.clone(), site_packages.clone()];
+        let search_paths = ModuleResolutionInputs {
+            extra_paths: vec![],
+            workspace_root: src.clone(),
+            site_packages: Some(site_packages.clone()),
+            custom_typeshed: Some(custom_typeshed.clone()),
+        };
 
         let mut db = TestDb::default();
-        set_module_search_paths(&mut db, roots);
+        set_module_search_paths(&mut db, search_paths);
 
         Ok(TestCase {
             temp_dir,
             db,
             src,
+            custom_typeshed,
             site_packages,
         })
     }
@@ -744,7 +836,7 @@ mod tests {
             ..
         } = create_resolver()?;
 
-        let foo_path = src.path().join("foo.py");
+        let foo_path = src.join("foo.py");
         std::fs::write(&foo_path, "print('Hello, world!')")?;
 
         let foo_module = resolve_module(&db, ModuleName::new("foo"))?.unwrap();
@@ -755,7 +847,7 @@ mod tests {
         );
 
         assert_eq!(ModuleName::new("foo"), foo_module.name(&db)?);
-        assert_eq!(&src, foo_module.path(&db)?.root());
+        assert_eq!(&src, foo_module.path(&db)?.root().path());
         assert_eq!(ModuleKind::Module, foo_module.kind(&db)?);
         assert_eq!(&foo_path, &*db.file_path(foo_module.path(&db)?.file()));
 
@@ -764,6 +856,98 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn stdlib() -> anyhow::Result<()> {
+        let TestCase {
+            db,
+            custom_typeshed,
+            ..
+        } = create_resolver()?;
+        let stdlib_dir = custom_typeshed.join(TYPESHED_STDLIB_DIRECTORY);
+        std::fs::create_dir_all(&stdlib_dir).unwrap();
+        let functools_path = stdlib_dir.join("functools.py");
+        std::fs::write(&functools_path, "def update_wrapper(): ...").unwrap();
+        let functools_module = resolve_module(&db, ModuleName::new("functools"))?.unwrap();
+
+        assert_eq!(
+            Some(functools_module),
+            resolve_module(&db, ModuleName::new("functools"))?
+        );
+        assert_eq!(&stdlib_dir, functools_module.path(&db)?.root().path());
+        assert_eq!(ModuleKind::Module, functools_module.kind(&db)?);
+        assert_eq!(
+            &functools_path,
+            &*db.file_path(functools_module.path(&db)?.file())
+        );
+
+        assert_eq!(
+            Some(functools_module),
+            path_to_module(&db, &functools_path)?
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn first_party_precedence_over_stdlib() -> anyhow::Result<()> {
+        let TestCase {
+            db,
+            src,
+            custom_typeshed,
+            ..
+        } = create_resolver()?;
+
+        let stdlib_dir = custom_typeshed.join(TYPESHED_STDLIB_DIRECTORY);
+        std::fs::create_dir_all(&stdlib_dir).unwrap();
+        std::fs::create_dir_all(&src).unwrap();
+
+        let stdlib_functools_path = stdlib_dir.join("functools.py");
+        let first_party_functools_path = src.join("functools.py");
+        std::fs::write(stdlib_functools_path, "def update_wrapper(): ...").unwrap();
+        std::fs::write(&first_party_functools_path, "def update_wrapper(): ...").unwrap();
+        let functools_module = resolve_module(&db, ModuleName::new("functools"))?.unwrap();
+
+        assert_eq!(
+            Some(functools_module),
+            resolve_module(&db, ModuleName::new("functools"))?
+        );
+        assert_eq!(&src, functools_module.path(&db).unwrap().root().path());
+        assert_eq!(ModuleKind::Module, functools_module.kind(&db)?);
+        assert_eq!(
+            &first_party_functools_path,
+            &*db.file_path(functools_module.path(&db)?.file())
+        );
+
+        assert_eq!(
+            Some(functools_module),
+            path_to_module(&db, &first_party_functools_path)?
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn typeshed_zip_created_at_build_time() -> anyhow::Result<()> {
+        // The file path here is hardcoded in this crate's `build.rs` script.
+        // Luckily this crate will fail to build if this file isn't available at build time.
+        const TYPESHED_ZIP_BYTES: &[u8] =
+            include_bytes!(concat!(env!("OUT_DIR"), "/zipped_typeshed.zip"));
+        assert!(!TYPESHED_ZIP_BYTES.is_empty());
+        let mut typeshed_zip_archive = ZipArchive::new(Cursor::new(TYPESHED_ZIP_BYTES))?;
+
+        let path_to_functools = Path::new("stdlib").join("functools.pyi");
+        let mut functools_module_stub = typeshed_zip_archive
+            .by_name(path_to_functools.to_str().unwrap())
+            .unwrap();
+        assert!(functools_module_stub.is_file());
+
+        let mut functools_module_stub_source = String::new();
+        functools_module_stub.read_to_string(&mut functools_module_stub_source)?;
+
+        assert!(functools_module_stub_source.contains("def update_wrapper("));
+        Ok(())
+    }
+
     #[test]
     fn resolve_package() -> anyhow::Result<()> {
         let TestCase {
@@ -773,7 +957,7 @@ mod tests {
             ..
         } = create_resolver()?;
 
-        let foo_dir = src.path().join("foo");
+        let foo_dir = src.join("foo");
         let foo_path = foo_dir.join("__init__.py");
         std::fs::create_dir(&foo_dir)?;
         std::fs::write(&foo_path, "print('Hello, world!')")?;
@@ -781,7 +965,7 @@ mod tests {
         let foo_module = resolve_module(&db, ModuleName::new("foo"))?.unwrap();
 
         assert_eq!(ModuleName::new("foo"), foo_module.name(&db)?);
-        assert_eq!(&src, foo_module.path(&db)?.root());
+        assert_eq!(&src, foo_module.path(&db)?.root().path());
         assert_eq!(&foo_path, &*db.file_path(foo_module.path(&db)?.file()));
 
         assert_eq!(Some(foo_module), path_to_module(&db, &foo_path)?);
@@ -801,17 +985,17 @@ mod tests {
             ..
         } = create_resolver()?;
 
-        let foo_dir = src.path().join("foo");
+        let foo_dir = src.join("foo");
         let foo_init = foo_dir.join("__init__.py");
         std::fs::create_dir(&foo_dir)?;
         std::fs::write(&foo_init, "print('Hello, world!')")?;
 
-        let foo_py = src.path().join("foo.py");
+        let foo_py = src.join("foo.py");
         std::fs::write(&foo_py, "print('Hello, world!')")?;
 
         let foo_module = resolve_module(&db, ModuleName::new("foo"))?.unwrap();
 
-        assert_eq!(&src, foo_module.path(&db)?.root());
+        assert_eq!(&src, foo_module.path(&db)?.root().path());
         assert_eq!(&foo_init, &*db.file_path(foo_module.path(&db)?.file()));
         assert_eq!(ModuleKind::Package, foo_module.kind(&db)?);
 
@@ -830,14 +1014,14 @@ mod tests {
             ..
         } = create_resolver()?;
 
-        let foo_stub = src.path().join("foo.pyi");
-        let foo_py = src.path().join("foo.py");
+        let foo_stub = src.join("foo.pyi");
+        let foo_py = src.join("foo.py");
         std::fs::write(&foo_stub, "x: int")?;
         std::fs::write(&foo_py, "print('Hello, world!')")?;
 
         let foo = resolve_module(&db, ModuleName::new("foo"))?.unwrap();
 
-        assert_eq!(&src, foo.path(&db)?.root());
+        assert_eq!(&src, foo.path(&db)?.root().path());
         assert_eq!(&foo_stub, &*db.file_path(foo.path(&db)?.file()));
 
         assert_eq!(Some(foo), path_to_module(&db, &foo_stub)?);
@@ -855,7 +1039,7 @@ mod tests {
             ..
         } = create_resolver()?;
 
-        let foo = src.path().join("foo");
+        let foo = src.join("foo");
         let bar = foo.join("bar");
         let baz = bar.join("baz.py");
 
@@ -866,7 +1050,7 @@ mod tests {
 
         let baz_module = resolve_module(&db, ModuleName::new("foo.bar.baz"))?.unwrap();
 
-        assert_eq!(&src, baz_module.path(&db)?.root());
+        assert_eq!(&src, baz_module.path(&db)?.root().path());
         assert_eq!(&baz, &*db.file_path(baz_module.path(&db)?.file()));
 
         assert_eq!(Some(baz_module), path_to_module(&db, &baz)?);
@@ -881,6 +1065,7 @@ mod tests {
             temp_dir: _,
             src,
             site_packages,
+            ..
         } = create_resolver()?;
 
         // From [PEP420](https://peps.python.org/pep-0420/#nested-namespace-packages).
@@ -896,14 +1081,14 @@ mod tests {
         //       two.py
         // ```
 
-        let parent1 = src.path().join("parent");
+        let parent1 = src.join("parent");
         let child1 = parent1.join("child");
         let one = child1.join("one.py");
 
         std::fs::create_dir_all(child1)?;
         std::fs::write(&one, "print('Hello, world!')")?;
 
-        let parent2 = site_packages.path().join("parent");
+        let parent2 = site_packages.join("parent");
         let child2 = parent2.join("child");
         let two = child2.join("two.py");
 
@@ -927,6 +1112,7 @@ mod tests {
             temp_dir: _,
             src,
             site_packages,
+            ..
         } = create_resolver()?;
 
         // Adopted test case from the [PEP420 examples](https://peps.python.org/pep-0420/#nested-namespace-packages).
@@ -942,7 +1128,7 @@ mod tests {
         //       two.py
         // ```
 
-        let parent1 = src.path().join("parent");
+        let parent1 = src.join("parent");
         let child1 = parent1.join("child");
         let one = child1.join("one.py");
 
@@ -950,7 +1136,7 @@ mod tests {
         std::fs::write(child1.join("__init__.py"), "print('Hello, world!')")?;
         std::fs::write(&one, "print('Hello, world!')")?;
 
-        let parent2 = site_packages.path().join("parent");
+        let parent2 = site_packages.join("parent");
         let child2 = parent2.join("child");
         let two = child2.join("two.py");
 
@@ -975,17 +1161,18 @@ mod tests {
             src,
             site_packages,
             temp_dir: _temp_dir,
+            ..
         } = create_resolver()?;
 
-        let foo_src = src.path().join("foo.py");
-        let foo_site_packages = site_packages.path().join("foo.py");
+        let foo_src = src.join("foo.py");
+        let foo_site_packages = site_packages.join("foo.py");
 
         std::fs::write(&foo_src, "")?;
         std::fs::write(&foo_site_packages, "")?;
 
         let foo_module = resolve_module(&db, ModuleName::new("foo"))?.unwrap();
 
-        assert_eq!(&src, foo_module.path(&db)?.root());
+        assert_eq!(&src, foo_module.path(&db)?.root().path());
         assert_eq!(&foo_src, &*db.file_path(foo_module.path(&db)?.file()));
 
         assert_eq!(Some(foo_module), path_to_module(&db, &foo_src)?);
@@ -1004,8 +1191,8 @@ mod tests {
             ..
         } = create_resolver()?;
 
-        let foo = src.path().join("foo.py");
-        let bar = src.path().join("bar.py");
+        let foo = src.join("foo.py");
+        let bar = src.join("bar.py");
 
         std::fs::write(&foo, "")?;
         std::os::unix::fs::symlink(&foo, &bar)?;
@@ -1015,12 +1202,12 @@ mod tests {
 
         assert_ne!(foo_module, bar_module);
 
-        assert_eq!(&src, foo_module.path(&db)?.root());
+        assert_eq!(&src, foo_module.path(&db)?.root().path());
         assert_eq!(&foo, &*db.file_path(foo_module.path(&db)?.file()));
 
         // Bar has a different name but it should point to the same file.
 
-        assert_eq!(&src, bar_module.path(&db)?.root());
+        assert_eq!(&src, bar_module.path(&db)?.root().path());
         assert_eq!(foo_module.path(&db)?.file(), bar_module.path(&db)?.file());
         assert_eq!(&foo, &*db.file_path(bar_module.path(&db)?.file()));
 
@@ -1039,7 +1226,7 @@ mod tests {
             ..
         } = create_resolver()?;
 
-        let foo_dir = src.path().join("foo");
+        let foo_dir = src.join("foo");
         let foo_path = foo_dir.join("__init__.py");
         let bar_path = foo_dir.join("bar.py");
 
