@@ -10,8 +10,8 @@ use crate::module::{resolve_module, ModuleName};
 use crate::parse::parse;
 use crate::semantic::types::{ModuleTypeId, Type};
 use crate::semantic::{
-    resolve_global_symbol, semantic_index, Definition, GlobalSymbolId, ImportDefinition,
-    ImportFromDefinition,
+    resolve_global_symbol, semantic_index, ConstrainedDefinition, Definition, GlobalSymbolId,
+    ImportDefinition, ImportFromDefinition,
 };
 use crate::{FileId, Name};
 
@@ -41,25 +41,47 @@ pub fn infer_symbol_public_type(db: &dyn SemanticDb, symbol: GlobalSymbolId) ->
     Ok(ty)
 }
 
-#[tracing::instrument(level = "trace", skip(db))]
-pub fn infer_type_from_definitions<T>(
+/// Infer type of a symbol as union of the given `Definitions`.
+fn infer_type_from_definitions<T>(
     db: &dyn SemanticDb,
     symbol: GlobalSymbolId,
     definitions: T,
 ) -> QueryResult<Type>
 where
-    T: Debug + Iterator<Item = Definition>,
+    T: Debug + IntoIterator<Item = Definition>,
+{
+    infer_type_from_constrained_definitions(
+        db,
+        symbol,
+        definitions
+            .into_iter()
+            .map(|definition| ConstrainedDefinition {
+                definition,
+                constraints: vec![],
+            }),
+    )
+}
+
+/// Infer type of a symbol as union of the given `ConstrainedDefinitions`.
+fn infer_type_from_constrained_definitions<T>(
+    db: &dyn SemanticDb,
+    symbol: GlobalSymbolId,
+    constrained_definitions: T,
+) -> QueryResult<Type>
+where
+    T: IntoIterator<Item = ConstrainedDefinition>,
 {
     let jar: &SemanticJar = db.jar()?;
-    let mut tys = definitions
-        .map(|def| infer_definition_type(db, symbol, def.clone()))
+    let mut tys = constrained_definitions
+        .into_iter()
+        .map(|def| infer_constrained_definition_type(db, symbol, def.clone()))
         .peekable();
     if let Some(first) = tys.next() {
         if tys.peek().is_some() {
-            Ok(Type::Union(jar.type_store.add_union(
+            Ok(jar.type_store.add_union(
                 symbol.file_id,
-                &Iterator::chain([first].into_iter(), tys).collect::<QueryResult<Vec<_>>>()?,
-            )))
+                &Iterator::chain(std::iter::once(first), tys).collect::<QueryResult<Vec<_>>>()?,
+            ))
         } else {
             first
         }
@@ -68,6 +90,37 @@ where
     }
 }
 
+/// Infer type for a ConstrainedDefinition (intersection of the definition type and the
+/// constraints)
+#[tracing::instrument(level = "trace", skip(db))]
+pub fn infer_constrained_definition_type(
+    db: &dyn SemanticDb,
+    symbol: GlobalSymbolId,
+    constrained_definition: ConstrainedDefinition,
+) -> QueryResult<Type> {
+    let ConstrainedDefinition {
+        definition,
+        constraints,
+    } = constrained_definition;
+    let index = semantic_index(db, symbol.file_id)?;
+    let parsed = parse(db.upcast(), symbol.file_id)?;
+    let mut intersected_types = vec![infer_definition_type(db, symbol, definition)?];
+    for constraint in constraints {
+        if let Some(constraint_type) = infer_constraint_type(
+            db,
+            symbol,
+            index.resolve_expression_id(parsed.syntax(), constraint),
+        )? {
+            intersected_types.push(constraint_type);
+        }
+    }
+    let jar: &SemanticJar = db.jar()?;
+    Ok(jar
+        .type_store
+        .add_intersection(symbol.file_id, &intersected_types, &[]))
+}
+
+/// Infer a type for a Definition
 #[tracing::instrument(level = "trace", skip(db))]
 pub fn infer_definition_type(
     db: &dyn SemanticDb,
@@ -79,7 +132,7 @@ pub fn infer_definition_type(
     let file_id = symbol.file_id;
 
     match definition {
-        Definition::None => Ok(Type::Unbound),
+        Definition::Unbound => Ok(Type::Unbound),
         Definition::Import(ImportDefinition {
             module: module_name,
         }) => {
@@ -122,7 +175,7 @@ pub fn infer_definition_type(
                     bases.push(infer_expr_type(db, file_id, base)?);
                 }
                 let scope_id = index.symbol_table().scope_id_for_node(node_key.erased());
-                let ty = Type::Class(type_store.add_class(file_id, &node.name.id, scope_id, bases));
+                let ty = type_store.add_class(file_id, &node.name.id, scope_id, bases);
                 type_store.cache_node_type(file_id, *node_key.erased(), ty);
                 Ok(ty)
             }
@@ -144,15 +197,13 @@ pub fn infer_definition_type(
                     .map(|decorator| infer_expr_type(db, file_id, &decorator.expression))
                     .collect::<QueryResult<_>>()?;
                 let scope_id = index.symbol_table().scope_id_for_node(node_key.erased());
-                let ty = type_store
-                    .add_function(
-                        file_id,
-                        &node.name.id,
-                        symbol.symbol_id,
-                        scope_id,
-                        decorator_tys,
-                    )
-                    .into();
+                let ty = type_store.add_function(
+                    file_id,
+                    &node.name.id,
+                    symbol.symbol_id,
+                    scope_id,
+                    decorator_tys,
+                );
                 type_store.cache_node_type(file_id, *node_key.erased(), ty);
                 Ok(ty)
             }
@@ -161,7 +212,7 @@ pub fn infer_definition_type(
             let parsed = parse(db.upcast(), file_id)?;
             let ast = parsed.syntax();
             let node = node_key.resolve_unwrap(ast.as_any_node_ref());
-            // TODO handle unpacking assignment correctly (here and for AnnotatedAssignment case, below)
+            // TODO handle unpacking assignment
             infer_expr_type(db, file_id, &node.value)
         }
         Definition::AnnotatedAssignment(node_key) => {
@@ -172,16 +223,74 @@ pub fn infer_definition_type(
             let Some(value) = &node.value else {
                 return Ok(Type::Unknown);
             };
-            // TODO handle unpacking assignment correctly (here and for Assignment case, above)
+            // TODO handle unpacking assignment
             infer_expr_type(db, file_id, value)
         }
+        Definition::NamedExpr(node_key) => {
+            let parsed = parse(db.upcast(), file_id)?;
+            let ast = parsed.syntax();
+            let node = node_key.resolve_unwrap(ast.as_any_node_ref());
+            infer_expr_type(db, file_id, &node.value)
+        }
     }
 }
 
+/// Return the type that the given constraint (an expression from a control-flow test) requires the
+/// given symbol to have. For example, returns the Type "~None" as the constraint type if given the
+/// symbol ID for x and the expression ID for `x is not None`. Returns (Rust) None if the given
+/// expression applies no constraints on the given symbol.
+#[tracing::instrument(level = "trace", skip(db))]
+fn infer_constraint_type(
+    db: &dyn SemanticDb,
+    symbol_id: GlobalSymbolId,
+    // TODO this should preferably take an &ast::Expr instead of AnyNodeRef
+    expression: ast::AnyNodeRef,
+) -> QueryResult<Option<Type>> {
+    let file_id = symbol_id.file_id;
+    let index = semantic_index(db, file_id)?;
+    let jar: &SemanticJar = db.jar()?;
+    let symbol_name = symbol_id.symbol_id.symbol(&index.symbol_table).name();
+    // TODO narrowing attributes
+    // TODO narrowing dict keys
+    // TODO isinstance, ==/!=, type(...), literals, bools...
+    match expression {
+        ast::AnyNodeRef::ExprCompare(ast::ExprCompare {
+            left,
+            ops,
+            comparators,
+            ..
+        }) => {
+            // TODO chained comparisons
+            match left.as_ref() {
+                ast::Expr::Name(ast::ExprName { id, .. }) if id == symbol_name => match ops[0] {
+                    ast::CmpOp::Is | ast::CmpOp::IsNot => {
+                        Ok(match infer_expr_type(db, file_id, &comparators[0])? {
+                            Type::None => Some(Type::None),
+                            _ => None,
+                        }
+                        .map(|ty| {
+                            if matches!(ops[0], ast::CmpOp::IsNot) {
+                                jar.type_store.add_intersection(file_id, &[], &[ty])
+                            } else {
+                                ty
+                            }
+                        }))
+                    }
+                    _ => Ok(None),
+                },
+                _ => Ok(None),
+            }
+        }
+        _ => Ok(None),
+    }
+}
+
+/// Infer type of the given expression.
 fn infer_expr_type(db: &dyn SemanticDb, file_id: FileId, expr: &ast::Expr) -> QueryResult<Type> {
     // TODO cache the resolution of the type on the node
     let index = semantic_index(db, file_id)?;
     match expr {
+        ast::Expr::NoneLiteral(_) => Ok(Type::None),
         ast::Expr::NumberLiteral(ast::ExprNumberLiteral { value, .. }) => {
             match value {
                 ast::Number::Int(n) => {
@@ -195,8 +304,7 @@ fn infer_expr_type(db: &dyn SemanticDb, file_id: FileId, expr: &ast::Expr) -> Qu
         ast::Expr::Name(name) => {
             // TODO look up in the correct scope, don't assume global
             if let Some(symbol_id) = index.symbol_table().root_symbol_id_by_name(&name.id) {
-                // TODO should use only reachable definitions, not public type
-                infer_type_from_definitions(
+                infer_type_from_constrained_definitions(
                     db,
                     GlobalSymbolId { file_id, symbol_id },
                     index.reachable_definitions(symbol_id, expr),
@@ -212,17 +320,35 @@ fn infer_expr_type(db: &dyn SemanticDb, file_id: FileId, expr: &ast::Expr) -> Qu
                 .get_member(db, attr_name)
                 .map(|ty| ty.unwrap_or(Type::Unknown))
         }
-        _ => todo!("full expression type resolution"),
+        ast::Expr::BinOp(ast::ExprBinOp {
+            left, op, right, ..
+        }) => {
+            let left_ty = infer_expr_type(db, file_id, left)?;
+            let right_ty = infer_expr_type(db, file_id, right)?;
+            // TODO add reverse bin op support if right <: left
+            left_ty.resolve_bin_op(db, *op, right_ty)
+        }
+        ast::Expr::Named(ast::ExprNamed { value, .. }) => infer_expr_type(db, file_id, value),
+        ast::Expr::If(ast::ExprIf { body, orelse, .. }) => {
+            // TODO detect statically known truthy or falsy test
+            let body_ty = infer_expr_type(db, file_id, body)?;
+            let else_ty = infer_expr_type(db, file_id, orelse)?;
+            let jar: &SemanticJar = db.jar()?;
+            Ok(jar.type_store.add_union(file_id, &[body_ty, else_ty]))
+        }
+        _ => todo!("expression type resolution for {:?}", expr),
     }
 }
 
 #[cfg(test)]
 mod tests {
 
+    use std::path::PathBuf;
+
     use crate::db::tests::TestDb;
     use crate::db::{HasJar, SemanticJar};
     use crate::module::{
-        resolve_module, set_module_search_paths, ModuleName, ModuleSearchPath, ModuleSearchPathKind,
+        resolve_module, set_module_search_paths, ModuleName, ModuleResolutionInputs,
     };
     use crate::semantic::{infer_symbol_public_type, resolve_global_symbol, Type};
     use crate::Name;
@@ -234,7 +360,7 @@ mod tests {
         temp_dir: tempfile::TempDir,
         db: TestDb,
 
-        src: ModuleSearchPath,
+        src: PathBuf,
     }
 
     fn create_test() -> std::io::Result<TestCase> {
@@ -242,18 +368,23 @@ mod tests {
 
         let src = temp_dir.path().join("src");
         std::fs::create_dir(&src)?;
-        let src = ModuleSearchPath::new(src.canonicalize()?, ModuleSearchPathKind::FirstParty);
+        let src = src.canonicalize()?;
 
-        let roots = vec![src.clone()];
+        let search_paths = ModuleResolutionInputs {
+            extra_paths: vec![],
+            workspace_root: src.clone(),
+            site_packages: None,
+            custom_typeshed: None,
+        };
 
         let mut db = TestDb::default();
-        set_module_search_paths(&mut db, roots);
+        set_module_search_paths(&mut db, search_paths);
 
         Ok(TestCase { temp_dir, db, src })
     }
 
     fn write_to_path(case: &TestCase, relative_path: &str, contents: &str) -> anyhow::Result<()> {
-        let path = case.src.path().join(relative_path);
+        let path = case.src.join(relative_path);
         std::fs::write(path, contents)?;
         Ok(())
     }
@@ -393,15 +524,13 @@ mod tests {
             ",
         )?;
 
-        assert_public_type(&case, "a", "x", "(Literal[1] | Literal[2])")
+        assert_public_type(&case, "a", "x", "Literal[1, 2]")
     }
 
     #[test]
     fn resolve_visible_def() -> anyhow::Result<()> {
         let case = create_test()?;
-
         write_to_path(&case, "a.py", "y = 1; y = 2; x = y")?;
-
         assert_public_type(&case, "a", "x", "Literal[2]")
     }
 
@@ -421,7 +550,7 @@ mod tests {
             ",
         )?;
 
-        assert_public_type(&case, "a", "x", "(Literal[2] | Literal[3])")
+        assert_public_type(&case, "a", "x", "Literal[2, 3]")
     }
 
     #[test]
@@ -438,7 +567,7 @@ mod tests {
             ",
         )?;
 
-        assert_public_type(&case, "a", "x", "(Unbound | Literal[1])")
+        assert_public_type(&case, "a", "x", "Literal[1] | Unbound")
     }
 
     #[test]
@@ -463,7 +592,7 @@ mod tests {
             ",
         )?;
 
-        assert_public_type(&case, "a", "x", "(Literal[3] | Literal[4] | Literal[5])")?;
+        assert_public_type(&case, "a", "x", "Literal[3, 4, 5]")?;
         assert_public_type(&case, "a", "r", "Literal[2]")?;
         assert_public_type(&case, "a", "s", "Literal[5]")
     }
@@ -486,6 +615,148 @@ mod tests {
             ",
         )?;
 
-        assert_public_type(&case, "a", "x", "(Literal[2] | Literal[3] | Literal[4])")
+        assert_public_type(&case, "a", "x", "Literal[2, 3, 4]")
+    }
+
+    #[test]
+    fn literal_int_arithmetic() -> anyhow::Result<()> {
+        let case = create_test()?;
+
+        write_to_path(
+            &case,
+            "a.py",
+            "
+                a = 2 + 1
+                b = a - 4
+                c = a * b
+                d = c / 3
+                e = 5 % 3
+            ",
+        )?;
+
+        assert_public_type(&case, "a", "a", "Literal[3]")?;
+        assert_public_type(&case, "a", "b", "Literal[-1]")?;
+        assert_public_type(&case, "a", "c", "Literal[-3]")?;
+        assert_public_type(&case, "a", "d", "Literal[-1]")?;
+        assert_public_type(&case, "a", "e", "Literal[2]")
+    }
+
+    #[test]
+    fn walrus() -> anyhow::Result<()> {
+        let case = create_test()?;
+
+        write_to_path(
+            &case,
+            "a.py",
+            "
+                x = (y := 1) + 1
+            ",
+        )?;
+
+        assert_public_type(&case, "a", "x", "Literal[2]")?;
+        assert_public_type(&case, "a", "y", "Literal[1]")
+    }
+
+    #[test]
+    fn ifexpr() -> anyhow::Result<()> {
+        let case = create_test()?;
+
+        write_to_path(
+            &case,
+            "a.py",
+            "
+                x = 1 if flag else 2
+            ",
+        )?;
+
+        assert_public_type(&case, "a", "x", "Literal[1, 2]")
+    }
+
+    #[test]
+    fn ifexpr_walrus() -> anyhow::Result<()> {
+        let case = create_test()?;
+
+        write_to_path(
+            &case,
+            "a.py",
+            "
+                y = z = 0
+                x = (y := 1) if flag else (z := 2)
+                a = y
+                b = z
+            ",
+        )?;
+
+        assert_public_type(&case, "a", "x", "Literal[1, 2]")?;
+        assert_public_type(&case, "a", "a", "Literal[0, 1]")?;
+        assert_public_type(&case, "a", "b", "Literal[0, 2]")
+    }
+
+    #[test]
+    fn ifexpr_walrus_2() -> anyhow::Result<()> {
+        let case = create_test()?;
+
+        write_to_path(
+            &case,
+            "a.py",
+            "
+                y = 0
+                (y := 1) if flag else (y := 2)
+                a = y
+            ",
+        )?;
+
+        assert_public_type(&case, "a", "a", "Literal[1, 2]")
+    }
+
+    #[test]
+    fn ifexpr_nested() -> anyhow::Result<()> {
+        let case = create_test()?;
+
+        write_to_path(
+            &case,
+            "a.py",
+            "
+                x = 1 if flag else 2 if flag2 else 3
+            ",
+        )?;
+
+        assert_public_type(&case, "a", "x", "Literal[1, 2, 3]")
+    }
+
+    #[test]
+    fn none() -> anyhow::Result<()> {
+        let case = create_test()?;
+
+        write_to_path(
+            &case,
+            "a.py",
+            "
+                x = 1 if flag else None
+            ",
+        )?;
+
+        assert_public_type(&case, "a", "x", "Literal[1] | None")
+    }
+
+    #[test]
+    fn narrow_none() -> anyhow::Result<()> {
+        let case = create_test()?;
+
+        write_to_path(
+            &case,
+            "a.py",
+            "
+                x = 1 if flag else None
+                y = 0
+                if x is not None:
+                    y = x
+                z = y
+            ",
+        )?;
+
+        // TODO normalization of unions and intersections: this type is technically correct but
+        // begging for normalization
+        assert_public_type(&case, "a", "z", "Literal[0] | Literal[1] | None & ~None")
     }
 }
