@@ -553,11 +553,6 @@ impl PrintedRange {
     pub fn source_range(&self) -> TextRange {
         self.source_range
     }
-
-    #[must_use]
-    pub fn with_code(self, code: String) -> Self {
-        Self { code, ..self }
-    }
 }
 
 /// Public return type of the formatter
@@ -780,10 +775,6 @@ where
         self.item = item;
         self
     }
-
-    pub fn into_item(self) -> T {
-        self.item
-    }
 }
 
 impl<T, R, C> Format<C> for FormatOwnedWithRule<T, R, C>
