@@ -20,6 +20,7 @@ pub(crate) use raise_literal::*;
 pub(crate) use raise_without_from_inside_except::*;
 pub(crate) use re_sub_positional_args::*;
 pub(crate) use redundant_tuple_in_exception_handler::*;
+pub(crate) use return_in_generator::*;
 pub(crate) use reuse_of_groupby_generator::*;
 pub(crate) use setattr_with_constant::*;
 pub(crate) use star_arg_unpacking_after_keyword_arg::*;
@@ -56,6 +57,7 @@ mod raise_literal;
 mod raise_without_from_inside_except;
 mod re_sub_positional_args;
 mod redundant_tuple_in_exception_handler;
+mod return_in_generator;
 mod reuse_of_groupby_generator;
 mod setattr_with_constant;
 mod star_arg_unpacking_after_keyword_arg;
