@@ -1,4 +1,5 @@
 use ruff_python_parser::{parse_unchecked, Mode};
+use ruff_python_trivia::CommentRanges;
 use ruff_source_file::Locator;
 use ruff_text_size::TextSize;
 
@@ -8,9 +9,10 @@ fn block_comments_two_line_block_at_start() {
     let source = "# line 1\n# line 2\n";
     let parsed = parse_unchecked(source, Mode::Module);
     let locator = Locator::new(source);
+    let comment_ranges = CommentRanges::from(parsed.tokens());
 
     // act
-    let block_comments = parsed.comment_ranges().block_comments(&locator);
+    let block_comments = comment_ranges.block_comments(&locator);
 
     // assert
     assert_eq!(block_comments, vec![TextSize::new(0), TextSize::new(9)]);
@@ -22,9 +24,10 @@ fn block_comments_indented_block() {
     let source = "    # line 1\n    # line 2\n";
     let parsed = parse_unchecked(source, Mode::Module);
     let locator = Locator::new(source);
+    let comment_ranges = CommentRanges::from(parsed.tokens());
 
     // act
-    let block_comments = parsed.comment_ranges().block_comments(&locator);
+    let block_comments = comment_ranges.block_comments(&locator);
 
     // assert
     assert_eq!(block_comments, vec![TextSize::new(4), TextSize::new(17)]);
@@ -36,9 +39,10 @@ fn block_comments_single_line_is_not_a_block() {
     let source = "\n";
     let parsed = parse_unchecked(source, Mode::Module);
     let locator = Locator::new(source);
+    let comment_ranges = CommentRanges::from(parsed.tokens());
 
     // act
-    let block_comments = parsed.comment_ranges().block_comments(&locator);
+    let block_comments = comment_ranges.block_comments(&locator);
 
     // assert
     assert_eq!(block_comments, Vec::<TextSize>::new());
@@ -50,9 +54,10 @@ fn block_comments_lines_with_code_not_a_block() {
     let source = "x = 1  # line 1\ny = 2  # line 2\n";
     let parsed = parse_unchecked(source, Mode::Module);
     let locator = Locator::new(source);
+    let comment_ranges = CommentRanges::from(parsed.tokens());
 
     // act
-    let block_comments = parsed.comment_ranges().block_comments(&locator);
+    let block_comments = comment_ranges.block_comments(&locator);
 
     // assert
     assert_eq!(block_comments, Vec::<TextSize>::new());
@@ -64,9 +69,10 @@ fn block_comments_sequential_lines_not_in_block() {
     let source = "    # line 1\n        # line 2\n";
     let parsed = parse_unchecked(source, Mode::Module);
     let locator = Locator::new(source);
+    let comment_ranges = CommentRanges::from(parsed.tokens());
 
     // act
-    let block_comments = parsed.comment_ranges().block_comments(&locator);
+    let block_comments = comment_ranges.block_comments(&locator);
 
     // assert
     assert_eq!(block_comments, Vec::<TextSize>::new());
@@ -83,9 +89,10 @@ fn block_comments_lines_in_triple_quotes_not_a_block() {
         "#;
     let parsed = parse_unchecked(source, Mode::Module);
     let locator = Locator::new(source);
+    let comment_ranges = CommentRanges::from(parsed.tokens());
 
     // act
-    let block_comments = parsed.comment_ranges().block_comments(&locator);
+    let block_comments = comment_ranges.block_comments(&locator);
 
     // assert
     assert_eq!(block_comments, Vec::<TextSize>::new());
@@ -119,9 +126,10 @@ y = 2  # do not form a block comment
         "#;
     let parsed = parse_unchecked(source, Mode::Module);
     let locator = Locator::new(source);
+    let comment_ranges = CommentRanges::from(parsed.tokens());
 
     // act
-    let block_comments = parsed.comment_ranges().block_comments(&locator);
+    let block_comments = comment_ranges.block_comments(&locator);
 
     // assert
     assert_eq!(
