@@ -1,8 +1,12 @@
+use std::borrow::Cow;
+use std::path::PathBuf;
+use std::{collections::BTreeMap, path::Path, sync::Arc};
+
 use anyhow::anyhow;
 use lsp_types::Url;
 use rustc_hash::FxHashMap;
-use std::path::PathBuf;
-use std::{collections::BTreeMap, path::Path, sync::Arc};
+
+pub(crate) use ruff_settings::RuffSettings;
 
 use crate::{
     edit::{DocumentKey, DocumentVersion, NotebookDocument},
@@ -13,8 +17,6 @@ use super::{settings::ResolvedClientSettings, ClientSettings};
 
 mod ruff_settings;
 
-pub(crate) use ruff_settings::RuffSettings;
-
 type SettingsIndex = BTreeMap<PathBuf, WorkspaceSettings>;
 
 /// Stores and tracks all open documents in a session, along with their associated settings.
@@ -185,6 +187,21 @@ impl Index {
         Self::register_workspace_settings(&mut self.settings, url, None, global_settings)
     }
 
+    pub(super) fn num_documents(&self) -> usize {
+        self.documents.len()
+    }
+
+    pub(super) fn num_workspaces(&self) -> usize {
+        self.settings.len()
+    }
+
+    pub(super) fn list_config_files(&self) -> Vec<&Path> {
+        self.settings
+            .values()
+            .flat_map(|WorkspaceSettings { ruff_settings, .. }| ruff_settings.list_files())
+            .collect()
+    }
+
     fn register_workspace_settings(
         settings_index: &mut SettingsIndex,
         workspace_url: &Url,
@@ -544,8 +561,10 @@ impl DocumentQuery {
     /// Get the path for the document selected by this query, ignoring whether the file exists on disk.
     ///
     /// Returns the URL's path if this is an unsaved (untitled) document.
-    pub(crate) fn virtual_file_path(&self) -> &Path {
-        Path::new(self.file_url().path())
+    pub(crate) fn virtual_file_path(&self) -> Cow<Path> {
+        self.file_path()
+            .map(Cow::Owned)
+            .unwrap_or_else(|| Cow::Borrowed(Path::new(self.file_url().path())))
     }
 
     /// Attempt to access the single inner text document selected by the query.
